close all
clear all
clc


load pedestrianDataset
% xyz dataset

N = size(xyz,2); % how many time samples?

Ts = 1e-2; % 10 ms sampling time

mn_pos_std = 75; % standard deviation [mm]

pn_pos_std = 5;
pn_speed_std = 25;
pn_acc_std = 125; 

P1_pos_std = mn_pos_std;
P1_speed_std = 100;
P1_acc_std = 150;

% ------------------------------
% system model equations
%
% x(k+1) = F x(k) + v1(k)
%   y(k) = H x(k) + v2(k) 
%
% v1(k), v2(k) i.i.d. gaussian white noises
%
% state vector x(k)
% 
% x(k) = [px(k) py(k) pz(k) vx(k) vy(k) vz(k) ax(k) ay(k) az(k)]'
%
% px py pz --> position 3D components
% vx vy vz --> speed vector components
% ax ay az --> acceleration vector components
% 

% --  state evolution matrix F --------------------------
F = [ 1  0  0  Ts  0  0  0.5*Ts*Ts    0        0; ...
      0  1  0  0  Ts  0      0     0.5*Ts*Ts   0; ...
      0  0  1  0   0  Ts     0        0   0.5*Ts*Ts; ...
      0  0  0  1   0  0      Ts       0        0; ...
      0  0  0  0   1  0      0        Ts       0; ...
      0  0  0  0   0  1      0        0        Ts; ...
      0  0  0  0   0  0      1        0        0; ...
      0  0  0  0   0  0      0        1        0; ...
      0  0  0  0   0  0      0        0        1];
  
  
% -- measurements matrix H -------------------------------
H = [ 1  0  0  0   0  0      0        0        0; ...
      0  1  0  0   0  0      0        0        0; ...
      0  0  1  0   0  0      0        0        0];
  
  
% -- process noise variance matrix V1 --------------------
V1 = diag([pn_pos_std; pn_pos_std; pn_pos_std; ...
            pn_speed_std; pn_speed_std; pn_speed_std; ...
            pn_acc_std; pn_acc_std; pn_acc_std]);


% -- measurement noise variance matrix V2 ----------------
V2 = diag([mn_pos_std; mn_pos_std; mn_pos_std]);

 
% -- state prediction error covariance matrix: initial value ----
P1 = diag([P1_pos_std; P1_pos_std; P1_pos_std;...
            P1_speed_std; P1_speed_std; P1_speed_std; ...
            P1_acc_std; P1_acc_std; P1_acc_std]);


% ========= time varying Kalman filter as filter =========================
xhat1_ = [xyz(:, 1);zeros(6,1)]; % 1st state a priori estimation
xhat = zeros(9,N); % output of the Kalman filter
e = zeros(size(xyz)); % innovation 
y = xyz; % the measurements
P = P1; % 1st P matrix

for k = 1:N
    
    e(:,k) = y(:,k) - H*xhat1_;
    K0 = (P*H')/(H*P*H'+V2);
    xhat(:,k) = xhat1_ + K0*e(:,k);
    Pnew = V1+F*(P-(P*H'/(V2+H*P*H'))*H*P)*F;
    P = Pnew;
    xhat1_ = F * xhat(:,k);
    
end % for k 


figure('Name', 'Traiettoria 2-D del pedone'); 
plot(xyz(1,:),xyz(2,:),'-o' );
axis square; grid on
% visualizza i dati in 2D
xlabel('coordinata X [mm]'); ylabel('coordinata Y [mm]');
% in evidenza il punto iniziale, in verde
hold on; plot(xyz(1,1),xyz(2,1),...
    'sg', 'MarkerSize', 12, 'MarkerFaceColor', 'g');
% in evidenza il punto di arrivo, in rosso
plot(xyz(1,end),xyz(2,end),...
    'dr', 'MarkerSize', 12, 'MarkerFaceColor', 'r');
set(gca, 'FontSize', 16, 'FontWeight', 'bold');
plot(xhat(1,:),xhat(2,:),'m*' );


figure('Name', 'Traiettoria 3-D del pedone'); 
plot3(xyz(1,:),xyz(2,:),xyz(3,:), '-o' );
axis equal; grid on
% visualizza i dati in 3D
xlabel('coordinata X [mm]'); 
ylabel('coordinata Y [mm]');
zlabel('coordinata Z [mm]');
% in evidenza il punto iniziale, in verde
hold on; plot3(xyz(1,1), xyz(2,1), xyz(3,1),...
    'sg', 'MarkerSize', 12, 'MarkerFaceColor', 'g');
% in evidenza il punto di arrivo, in rosso
plot3(xyz(1,end), xyz(2,end), xyz(3,end),...
    'dr', 'MarkerSize', 12, 'MarkerFaceColor', 'r');
set(gca, 'FontSize', 16, 'FontWeight', 'bold');


      